(function (app) {

    app.localize = function (key) {
        return translations[key];
    };

    var translations = {
        'changes are saved': 'All changes are saved',
        'changes are not saved': 'Changes have NOT been saved. Please reload the page and change the settings again. Contact support@easygenerator.com if problem persists.',
        'settings are not initialize': 'Template settings are not initialized. Please reload the page and change the settings again. Contact support@easygenerator.com if problem persists.',

        //graphical customization tab
        'graphical customization': 'Graphical customization',
		
		'cartoon': 'Cartoon light',
        'grey': 'Grey scheme',
        'black': 'Black scheme',
        'flat': 'Flat scheme',
		
        'custom course logo': 'Custom course logo:',
        'or': 'or',
        'upload logo image': 'Upload logo image',
        'upgrade account hint': 'You have to <a target="_blank" href="/account/upgrade">upgrade your account</a> in order to set custom course logo',
        'recomend size': '- recommended size 300x50 px, allowed formats: jpeg, jpg, png, bmp, gif',

        'logo update': 'Change',
        'logo remove': 'Clear',

        'choose color scheme': 'Choose color scheme:',
        'scheme': 'Scheme:',

        //background
        'background change': 'Change background:',
        'background update': 'Change background',
        'background remove': 'Clear background',
        'background original': 'Original',
        'background repeat': 'Repeat',
        'background fullscreen': 'Fullscreen',
        'background upgrade account hint': 'You have to <a target="_blank" href="/account/upgrade">upgrade your account</a> in order to set custom course background',
        //choose objective style
        'Choose objective style:': 'Choose objectives list style:',
        //toc
        'Display table of content:': 'Display table of content:',
        'Dont show': 'Don\'t show',
        'Show': 'Show'
    };

})(window.app = window.app || {});

(function () {
    ko.bindingHandlers.localize = {
        init: function (element, valueAccessor) {
            var $element = $(element),
                key = ko.unwrap(valueAccessor());

            $element.html(app.localize(key));
        }
    };

    ko.bindingHandlers.switchToggle = {
        init: function (element, valueAccessor) {
            var switchToggle = ko.bindingHandlers.switchToggle,
                viewModel = switchToggle.viewModel(element, valueAccessor),
                value = ko.unwrap(valueAccessor().value());

            viewModel.setInitialValue(value);

            switchToggle.onClick(element, function () {
                viewModel.toggle();

                var currentValue = ko.unwrap(valueAccessor().value());
                valueAccessor().value(!currentValue);
            });
        },
        update: function (element, valueAccessor) {
            var viewModel = ko.bindingHandlers.switchToggle.viewModel(element, valueAccessor),
                value = ko.unwrap(valueAccessor().value());

            viewModel.updateValue(value);
        },
        viewModel: function (element) {
            var $element = $(element),
                $wrapper = $('.switch-toggle-wrapper', $element);

            function setInitialValue(value) {
                setElementValue(value);
                updateElementPosition(value);
            }

            function toggle() {
                var value = getValue();
                setElementValue(!value);

                $wrapper.stop().animate({
                    marginLeft: calculateElementLeftMargin(!value)
                }, 250);
            }

            function getValue() {
                return $element.hasClass('on');
            }

            function updateValue(value) {
                if (getValue() != value) {
                    setInitialValue(value);
                }
            }

            function setElementValue(value) {
                $element.toggleClass('on', value);
                $element.toggleClass('off', !value);
            }

            function updateElementPosition(value) {
                $wrapper.css('margin-left', calculateElementLeftMargin(value) + 'px');
            }

            function calculateElementLeftMargin(value) {
                return value ? 0 : $element.height() - $element.width();
            }

            return {
                setInitialValue: setInitialValue,
                updateValue: updateValue,
                toggle: toggle
            }
        },
        onClick: function (element, handler) {
            var $element = $(element),
                isMouseDownFired = false;

            $element.mousedown(function (event) {
                if (event.which != 1)
                    return;

                isMouseDownFired = true;
                handler();
            });

            $element.click(function () {
                if (isMouseDownFired) {
                    isMouseDownFired = false;
                    return;
                }

                handler();
            });
        }
    };
})();

(function (app) {
    app.SectionsPageModel = SectionsPageModel;
    app.TreeOfContentModel = TreeOfContentModel;
    app.SectionsLayoutModel = SectionsLayoutModel;

    function SectionsPageModel(settings, saveChanges) {
        var that = this;

        that.enabled = ko.observable(true);
        that.getData = getData;
        init(settings);

        function init(settings) {
            if (settings && settings.enabled !== undefined) {
                that.enabled(settings.enabled);
            }

            that.enabled.subscribe(function () {
                saveChanges();
            });
        }

        function getData() {
            return {
                enabled: that.enabled()
            }
        }
    }

    function TreeOfContentModel(settings, saveChanges) {
        var that = this;

        that.enabled = ko.observable(true);
        that.getData = getData;
        init(settings);

        function init(settings) {
            if (settings) {
                that.enabled(settings.enabled);
            }

            that.enabled.subscribe(function () {
                saveChanges();
            });
        }

        function getData() {
            return {
                enabled: that.enabled()
            }
        }
    }

    function SectionsLayoutModel(layoutSettings, saveChanges) {
        var that = this;

        that.list = [
            new LayoutModel('Tiles', true),
            new LayoutModel('List'),
        ];

        that.selectedLayoutName = ko.computed(function () {
            var selectedName = '';
            ko.utils.arrayForEach(that.list, function (layout) { //foreach because of we need to track selecting of all themes
                if (layout.isSelected()) {
                    selectedName = layout.name;
                }
            });
            return selectedName;
        }, that);

        that.select = select;
        that.selectByName = selectByName;
        that.getData = getData;

        init(layoutSettings);

        return that;

        function init(layoutSettings) {
            if (!layoutSettings || !layoutSettings.key) {
                return;
            }
            that.selectByName(layoutSettings.key);
        }

        function select(item) {
            ko.utils.arrayForEach(that.list, function (layout) {
                layout.isSelected(false);
            });

            item.isSelected(true);

            saveChanges();
        }

        function selectByName(name) {
            ko.utils.arrayForEach(that.list, function (layout) {
                layout.isSelected(layout.name === name);
            });

        }

        function getData() {
            
            return {
                key: that.selectedLayoutName()
            };
        }

        function LayoutModel(name, isSelected) {
            var that = this;

            that.name = name;
            that.isSelected = ko.observable(isSelected === true);

            return that;
        }
    }

})(window.app = window.app || {});

(function (app) {

    var
        currentSettings = null,
        currentExtraData = null;

    var viewModel = {
        isError: ko.observable(false),

        sectionsPage: null,
        treeOfContent: null,
        sectionsLayout:null
    };

    viewModel.getCurrentSettingsData = function (settings) {
        return $.extend({}, settings || currentSettings, {
            treeOfContent: viewModel.treeOfContent.getData(),
            sectionsLayout: viewModel.sectionsLayout.getData(),
            sectionsPage: viewModel.sectionsPage.getData()
        });
    };

    viewModel.getCurrentExtraData = function () {
        return {};
    };

    viewModel.saveChanges = function () {
       
        var settings = viewModel.getCurrentSettingsData(),
            extraData = viewModel.getCurrentExtraData(),
            newSettings = JSON.stringify(settings),
            newExtraData = JSON.stringify(extraData);
        
        if (JSON.stringify(currentSettings) === newSettings && JSON.stringify(currentExtraData) === newExtraData) {
            return;
        }

        window.egApi.saveDesignSettings(newSettings, newExtraData, app.localize('changes are saved'), app.localize('changes are not saved'))
            .done(function () {
                currentSettings = settings;
                currentExtraData = extraData;
            });
    };

    viewModel.init = function () {
        var api = window.egApi;
        return api.init().then(function () {
            var manifest = api.getManifest();

            var defaultSettings = manifest.defaultTemplateSettings;
            var settings = api.getDesignSettings();

            viewModel.sectionsLayout = new app.SectionsLayoutModel(settings.sectionsLayout || defaultSettings.sectionsLayout, viewModel.saveChanges);
            viewModel.treeOfContent = new app.TreeOfContentModel(settings.treeOfContent || defaultSettings.treeOfContent, viewModel.saveChanges);
            viewModel.sectionsPage = new app.SectionsPageModel(settings.sectionsPage || defaultSettings.sectionsPage, viewModel.saveChanges);

            currentSettings = viewModel.getCurrentSettingsData(settings);
            currentExtraData = viewModel.getCurrentExtraData();

        }).fail(function () {
            viewModel.isError(true);
        });
    };

    viewModel.init().always(function () {
        $(document).ready(function () {
            ko.applyBindings(viewModel, $('.settings-container')[0]);
            $(window).on('blur', viewModel.saveChanges);
        });
    });

})(window.app = window.app || {});
